/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef JAC_AP_VEDSERVICE_VEDSERVICE_PROXY_H
#define JAC_AP_VEDSERVICE_VEDSERVICE_PROXY_H

#include "ara/com/internal/proxy/proxy_adapter.h"
#include "ara/com/internal/proxy/event_adapter.h"
#include "ara/com/internal/proxy/field_adapter.h"
#include "ara/com/internal/proxy/method_adapter.h"
#include "ara/com/crc_verification.h"
#include "jac/ap/vedservice/vedservice_common.h"
#include <string>

namespace jac {
namespace ap {
namespace vedservice {
namespace proxy {
namespace events {
    using VED4DPCEvent = ara::com::internal::proxy::event::EventAdapter<::dpc::VED4DPC>;
    using VED2FusionEgoMotionEvent = ara::com::internal::proxy::event::EventAdapter<::dpc::VED4FusionEgoMotion>;
    static constexpr ara::com::internal::EntityId VEDServiceVED4DPCEventId = 55898U; //VED4DPCEvent_event_hash
    static constexpr ara::com::internal::EntityId VEDServiceVED2FusionEgoMotionEventId = 57422U; //VED2FusionEgoMotionEvent_event_hash
}

namespace fields {
}

namespace methods {

} // namespace methods

class VEDServiceProxy {
private:
    std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
public:
    using HandleType = ara::com::HandleType;
    class ConstructionToken {
    public:
        explicit ConstructionToken(const std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> & proxy)
            : proxyAdapter(proxy),
              VED4DPCEvent(proxyAdapter->GetProxy(), events::VEDServiceVED4DPCEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
              VED2FusionEgoMotionEvent(proxyAdapter->GetProxy(), events::VEDServiceVED2FusionEgoMotionEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

        ConstructionToken(ConstructionToken&& other) = default;
        ConstructionToken& operator=(ConstructionToken && other) = default;

        ConstructionToken(const ConstructionToken&) = delete;
        ConstructionToken& operator = (const ConstructionToken&) = delete;
        
        virtual ~ConstructionToken() = default;

        /* Do not use such internal interfaces!!! */
        ara::core::Result<void> InitializeAll()
        {
            ara::core::Result<void> initResult;
            return initResult;
        }
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> GetProxyAdapter()
        {
            return proxyAdapter;
        }
        events::VED4DPCEvent GetVED4DPCEvent() noexcept { return std::move(VED4DPCEvent); }
        events::VED2FusionEgoMotionEvent GetVED2FusionEgoMotionEvent() noexcept { return std::move(VED2FusionEgoMotionEvent); }
    private:
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> proxyAdapter;
        events::VED4DPCEvent VED4DPCEvent;
        events::VED2FusionEgoMotionEvent VED2FusionEgoMotionEvent;
    };

    virtual ~VEDServiceProxy()
    {
        VED4DPCEvent.UnsetReceiveHandler();
        VED4DPCEvent.Unsubscribe();
        VED2FusionEgoMotionEvent.UnsetReceiveHandler();
        VED2FusionEgoMotionEvent.Unsubscribe();

    }

    explicit VEDServiceProxy(const HandleType &handle)
        : proxyAdapter(std::make_shared<ara::com::internal::proxy::ProxyAdapter>(::jac::ap::vedservice::VEDService::ServiceIdentifier, handle)),
          VED4DPCEvent(proxyAdapter->GetProxy(), events::VEDServiceVED4DPCEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()),
          VED2FusionEgoMotionEvent(proxyAdapter->GetProxy(), events::VEDServiceVED2FusionEgoMotionEventId, proxyAdapter->GetHandle(), proxyAdapter->GetServiceInterfaceInfo()){
        }

    void ThrowError(const ara::core::Result<void>& result) const
    {
        if (!(result.HasValue())) {
#ifndef NOT_SUPPORT_EXCEPTIONS
            ara::core::ErrorCode errorcode(result.Error());
            throw ara::com::ComException(std::move(errorcode));
#else
            std::cerr << "Error: Not support exception, create proxy failed!"<< std::endl;
#endif
        }
    }

    VEDServiceProxy(const VEDServiceProxy&) = delete;
    VEDServiceProxy& operator=(const VEDServiceProxy&) = delete;

    VEDServiceProxy(VEDServiceProxy&&) = default;
    VEDServiceProxy& operator=(VEDServiceProxy&&) = default;
    VEDServiceProxy(ConstructionToken&& token) noexcept
        : proxyAdapter(token.GetProxyAdapter()),
          VED4DPCEvent(token.GetVED4DPCEvent()),
          VED2FusionEgoMotionEvent(token.GetVED2FusionEgoMotionEvent()){
        static_cast<void>(std::move(token));
    }

    static ara::core::Result<ConstructionToken> Preconstruct(
        const ara::com::HandleType &handle)
    {
        std::shared_ptr<ara::com::internal::proxy::ProxyAdapter> preProxyAdapter =
            std::make_shared<ara::com::internal::proxy::ProxyAdapter>(
               ::jac::ap::vedservice::VEDService::ServiceIdentifier, handle);
        ConstructionToken token{std::move(preProxyAdapter)};
        ara::core::Result<void> initResult {token.InitializeAll()};
        if (initResult.HasValue()) {
            return ara::core::Result<ConstructionToken>(std::move(token));
        } else {
            const ara::core::ErrorCode errorcode{initResult.Error()};
            ara::core::Result<VEDServiceProxy::ConstructionToken> preResult{errorcode};
            return preResult;
        }
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType>& handler,
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::vedservice::VEDService::ServiceIdentifier, instance);
    }

    static ara::com::FindServiceHandle StartFindService(
        const ara::com::FindServiceHandler<ara::com::internal::proxy::ProxyAdapter::HandleType> handler,
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::StartFindService(handler, ::jac::ap::vedservice::VEDService::ServiceIdentifier, specifier);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::com::InstanceIdentifier instance)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::vedservice::VEDService::ServiceIdentifier, instance);
    }

    static ara::com::ServiceHandleContainer<ara::com::internal::proxy::ProxyAdapter::HandleType> FindService(
        const ara::core::InstanceSpecifier specifier)
    {
        return ara::com::internal::proxy::ProxyAdapter::FindService(::jac::ap::vedservice::VEDService::ServiceIdentifier, specifier);
    }

    static void StopFindService(const ara::com::FindServiceHandle& handle)
    {
        ara::com::internal::proxy::ProxyAdapter::StopFindService(handle);
    }

    HandleType GetHandle() const
    {
        return proxyAdapter->GetHandle();
    }
    bool SetEventThreadNumber(const std::uint16_t number, const std::uint16_t queueSize)
    {
        return proxyAdapter->SetEventThreadNumber(number, queueSize);
    }
    events::VED4DPCEvent VED4DPCEvent;
    events::VED2FusionEgoMotionEvent VED2FusionEgoMotionEvent;
};
} // namespace proxy
} // namespace vedservice
} // namespace ap
} // namespace jac

#endif // JAC_AP_VEDSERVICE_VEDSERVICE_PROXY_H
